## Code for FSIS' 2024 Risk Assessments
#### Receiving Model


#Set the number of iterations for the Sampling part of the SIR method 
iter=1000000

obs.true=matrix(c(20,15,14,119), nrow=2)

p.true=(0.202+0.208)/2 #assumed true proportion of C1 serotypes
obs.true=matrix(c(20,15,14,119), nrow=2)
n.pop=sum(obs.true)

#define prior for the population
a.sim=rexp(iter,1)
b.sim=rexp(iter,1/4)


#simulation storage variable
c1.sim=matrix(rep(0,iter*n.pop), nrow=iter)
n1.rh=matrix(rep(0,iter*n.pop), nrow=iter)
n1.pc=matrix(rep(0,iter*n.pop), nrow=iter)
p.hat.sim=rep(0,iter)
p.value=rep(0,iter)
z.value=rep(0,iter)
up.corner=rep(0,iter)

for (i in 1:iter) {

   c1.sim[i,]=rbeta(n.pop,a.sim[i],b.sim[i])
   n1.rh[i,]=rbinom(n.pop,size=1,c1.sim[i,])
   n1.pc[i,]=rbinom(n.pop,size=1,c1.sim[i,])
   p.hat.sim[i]=sum(n1.rh[i,])/n.pop
   z.value[i]=1-pnorm(abs(p.hat.sim[i]-p.true)/sqrt(p.true*(1-p.true)/(n.pop-1)),0,1)
   colSums(cbind(n1.rh[i,],n1.pc[i,]))

   obs.sample=matrix(c(sum(ifelse(n1.rh[i,]==1 & n1.pc[i,]==1,1,0)),sum(ifelse(n1.rh[i,]==1 & n1.pc[i,]==0,1,0)),
                    sum(ifelse(n1.rh[i,]==0 & n1.pc[i,]==1,1,0)),sum(ifelse(n1.rh[i,]==0 & n1.pc[i,]==0,1,0))),nrow=2)
   up.corner[i]=sum(ifelse(n1.rh[i,]==1 & n1.pc[i,]==1,1,0))
   p.value[i]=1-pchisq(sum((obs.sample-obs.true)^2/obs.true),1)


}

#summary information from Sampling step
mean(p.value)
mean(z.value)

#weights based on likelihood the sample came from the observed distribution
w=p.value*z.value 

#Choose the fraction for the Resampling step
num.resam=round(iter/20)  

index=sample(seq(1,iter),num.resam,replace=T,prob=w)  #weighted wr sample
mean(p.value)
mean(p.value[index])

c1.sim.star=c1.sim[index,]  
n1.rh.star=n1.rh[index,]
n1.pc.star=n1.pc[index,]
a.star=a.sim[index]
b.star=b.sim[index]

#Compare the density of C1 serotypes from teh sampling and resampling distributions
plot(density(c1.sim), col="darkorange", xlab="fraction of Cluster 1 serotypes in a flock", xlim=c(0,1))
lines(density(c1.sim.star), col="blue")
quantile(c1.sim.star, c(0.05,0.1,0.2,0.3,0.4))
quantile(c1.sim, c(0.05,0.1,0.2,0.3,0.4))

#Check that the SIR rehang and post-chill numbers look about right

n1.n1.star=rep(0,num.resam)
n1.n2.star=rep(0,num.resam)
n2.n1.star=rep(0,num.resam)
n2.n2.star=rep(0,num.resam)

for (i in 1:num.resam) {

  n1.n1.star[i]=sum(ifelse(n1.rh.star[i,]==1 & n1.pc.star[i,]==1,1,0))
  n1.n2.star[i]=sum(ifelse(n1.rh.star[i,]==1 & n1.pc.star[i,]==0,1,0))
  n2.n1.star[i]=sum(ifelse(n1.rh.star[i,]==0 & n1.pc.star[i,]==1,1,0))
  n2.n2.star[i]=sum(ifelse(n1.rh.star[i,]==0 & n1.pc.star[i,]==0,1,0))
#  obs.sample=matrix(c(sum(ifelse(n1.rh.star[i,]==1 & n1.pc[i,]==1,1,0)),sum(ifelse(n1.rh[i,]==1 & n1.pc[i,]==0,1,0)),
#                      sum(ifelse(n1.rh[i,]==0 & n1.pc[i,]==1,1,0)),sum(ifelse(n1.rh[i,]==0 & n1.pc[i,]==0,1,0))),nrow=2)
#  p.value[i]=1-pchisq(sum((obs.sample-obs.true)^2/obs.true),1)
  #   print(p.value[i])
  
}
round(matrix(c(mean(n1.n1.star),
mean(n1.n2.star),
mean(n2.n1.star),
mean(n2.n2.star)), nrow=2),1)
rowSums(matrix(c(mean(n1.n1.star),
                 mean(n1.n2.star),
                 mean(n2.n1.star),
                 mean(n2.n2.star)), nrow=2))
colSums(matrix(c(mean(n1.n1.star),
                 mean(n1.n2.star),
                 mean(n2.n1.star),
                 mean(n2.n2.star)), nrow=2))

obs.true

summary(a.star)
summary(b.star)
cor(a.star,b.star)
plot(a.sim,b.sim, pch=".", main="Prior and posterior distribution of F_C1", xlab="a", ylab="b")
points(a.star,b.star, pch=1, col="red")
legend(8,20,legend=c("Prior","Posterior"), pch=c(1), col=c("black", "red"))


summary(a.star/(a.star+b.star))
mean(c1.sim)
mean(c1.sim.star)

x=seq(0.001,0.999,0.001)
plot(x, pbeta(x, mean(a.star), mean(b.star)), type="l", ylim=c(0,1), ylab="cumulative density", xlab="Frequency of F_C1", main="Cumulative density for the frequency of high virulence serotypes")


cbind(seq(0.05,0.999,0.05),pbeta(seq(0.05,0.999,0.05), mean(a.star), mean(b.star)))

#Make an example histogram for a small set of pretend establishment  
n.flocks=100
p.flock=rbeta(n.flocks, mean(a.star), mean(b.star))
hist(p.flock)

#Check thinking
p.i=rep(0,10000)

for (i in 1:10000){
  p.i[i]=sum(rbinom(n.flocks, size=1, prob=p.flock))/n.flocks
}

summary(p.i)
sum(dbinom(1, size=1, prob=p.flock))/n.flocks
sum(1-pbinom(0, size=1, prob=p.flock))/n.flocks



#check prob of each outcome for the first flock using 2 or more out of 4 for failing
dbinom(2:4, size=4, prob=p.flock[1])
1-pbinom(1, size=4, prob=p.flock[1])



