## Code for FSIS' 2024 Risk Assessments
## Code to analyze chicken carcass data for process control of indicator organisms

## The data necessary to implement this model has establishment identifying volume information and is therefore not provided.

#######################################
##### SET WORKING DIRECTORY       #####
#######################################

# Notes on files/etc

# Use 'setwd' to identify working directory where all files are saved
# e.g., setwd('Documents/FSIS_2024_Sal_Risk_Assessment_Model_Code/Chicken')


#######################################
##### Begin                       #####
#######################################


db_est=function(s,n,w) {
  ###############################################
  #  returns the prevalence estimate, variance, 
  #  coef. variation and confidence intervals for
  #  the estimators derived under the design-
  #  based inferential paradigm
  ###############################################
  
  p.estab=s/n
  p.hat=sum(w*p.estab)
  v.estab=p.estab*(1-p.estab)/(n-1) 
  v.hat=sum(w*w*v.estab)
  alpha.hat=( p.hat*(1-p.hat)/v.hat-1 )*p.hat
  beta.hat=( p.hat*(1-p.hat)/v.hat-1 )*(1-p.hat)
  dbeta
  
  return(cbind(p.hat=p.hat, v.hat=v.hat, cv.p.hat=sqrt(v.hat)/p.hat,lo.hat=qbeta(0.025,alpha.hat, beta.hat), up.hat=qbeta(0.975,alpha.hat, beta.hat)))
  
}


#dat=read.csv() ##Read in data if available. 
names(dat)
num.estab=length(dat$num.sam)

n=dat$num.sam
n=ifelse(dat$n.m.s.pc+dat$s.pc==1,1.5,dat$n.m.s.pc+dat$s.pc) #If needed add 1/2 for estab with 1 sample to do variance est
p=dat$s.pc/n
s=dat$s.pc
w=dat$w
print(db_est(dat$s.pc,n,w))
pop.params=db_est(s,n,w)

#example 2x2 table comparing perf. standard approaches
cut.off=3
pass.apc <- ifelse(dat$dif.cen.mean>cut.off,1,0)
pass.prev <- ifelse(dat$prev.pc<5/51,1,0)      
table(pass.apc,pass.prev)

#run through a range of cut-off values for the AC standard
steps=26
param.fail=matrix(0,nrow=steps,ncol=5)
param.pass=param.fail
cut.off=rep(0,steps)
omega=rep(0,steps)
num.pass=rep(0,steps)
alf=0.5 #alpha is fraction of establishment expected to become passing if compliance is optional 
prev.alpha=rep(0,steps) #new prevalence under "AC guidence" implementation
Se=rep(0,steps)
Sp=rep(0,steps)

for (i in 1:steps) {

   cut.off[i]=1+0.1*i
   passing=subset(dat, ifelse(dat$dif.cen.mean>=cut.off[i],T,F))
   Se[i]=sum(ifelse(dat$dif.cen.mean>=cut.off[i],1,0)*dat$pass.ps)/sum(pass.prev)
   Sp[i]=sum(ifelse(dat$dif.cen.mean<cut.off[i],1,0)*(1-dat$pass.ps))/sum(1-pass.prev)
   num.pass[i]=length(passing$s.pc)
   failing=subset(dat, ifelse(dat$dif.cen.mean>cut.off[i],F,T))
   omega[i]=sum(passing$w)/sum(dat$w)
   n=ifelse(dat$n.m.s.pc+dat$s.pc==1,1.5,dat$n.m.s.pc+dat$s.pc) #add 1/2 for estab with 1 sample to do
   s=dat$s.pc
   w=dat$w
   param.pass[i,]=db_est(passing$s.pc,ifelse(passing$num.sam==1,1.5,passing$num.sam), passing$w/sum(passing$w))
   param.fail[i,]=db_est(failing$s.pc,ifelse(failing$num.sam==1,1.5,failing$num.sam), failing$w/sum(failing$w))
   prev.alpha[i]=(omega[i]+alf-alf*omega[i])*param.pass[i,1] + (1-omega[i])*(1-alf)*param.fail[i,1]
#Se and Sp
   both.pass=sum(pass.apc*dat$pass.ps)
   print(cbind(cut.off[i],param.pass[i,1],param.fail[i,1], omega[i], num.pass[i]/num.estab,sum(passing$pass.ps)/length(passing$pass.ps)))
   
}   

plot(cut.off,param.pass[,1],ylab="prevalence in each class", xlab="Minimum AC log-reduction", ylim=c(0,0.1),col="darkgreen", main="Prevalence in passing and failing establishments
     that achive the minumum AC log10 reduction" )
points(cut.off,param.fail[,1],pch=2,col="red" )
lines(cut.off,param.pass[,1],col="darkgreen")
lines(cut.off,param.fail[,1],col="red")
points(cut.off,prev.alpha,pch=3,col="purple" )
lines(cut.off,prev.alpha,col="purple")
abline(v=2.9)
abline(v=3.3, col="purple")

cbind(cut.off,omega,num.pass/num.estab)


abline(h=pop.params[1])
abline(h=0.75*pop.params[1], lty=2)
legend("topright",legend=c("passing","failing","alpha=0.5"),pch=c(1,2,3),col=c("darkgreen","red","purple"))

plot(cut.off,omega,ylab="fraction passing",xlab="AC log10 reduction",ylim=c(0,1),col="darkgreen", main="Fraction of production volume and establishments
     that are already meeting AC log reduction goals" )
lines(cut.off,omega,col="green")
points(cut.off,num.pass/num.estab,pch=2,col="red" )
lines(cut.off,num.pass/num.estab,col="red" )
legend("bottomleft",legend=c("volume passing","prop. estabs passing"),pch=c(1,2),col=c("darkgreen","red"))
abline(v=2.9)
abline(v=3.3, col="purple")

cut.off[order(Se+Sp)]
Se[order(Se+Sp)]+Sp[order(Se+Sp)]




plot(cut.off,Se, ylim=c(0,1), col="darkgreen",ylab="Se/Sp",
     xlab="fraction of samples with no detected AC", main="Performance charcteristics of performance standards 
     based on fractions of AC samples below LOD")
lines(cut.off,Se, col="darkgreen")
points(cut.off,Sp, col="red")
lines(cut.off,Sp, col="red")
legend(1.4,0.22,legend=c("Se=passing both/passing ps","Sp=failing both/failing ps"),pch=c(1,2),col=c("darkgreen","red"), lty=c(1,1))
abline(v=.35, col="purple")

cbind(cut.off,Se,Sp,Se+Sp)


#Redo analysis using the fractions of non-detects in AC samples (i.e., <10 cfu/ml for current method) 

hist(dat$frac.cen.pc, main="Fraction of AC samples below LOD", xlab="fraction of non-detects", ylab="number of establishments (N=199)")

#Look at an example
cut.off=.2
pass.apc <- ifelse(dat$frac.cen.pc>cut.off,1,0)
pass.prev <- ifelse(dat$prev.pc<5/51,1,0)      
table(pass.apc,pass.prev)
chisq.test(pass.prev,pass.apc)

#iterate through an increasing series of the fraction of samples where no AC were detected
steps=13
param.fail=matrix(0,nrow=steps,ncol=5)
param.pass=param.fail
cut.off=rep(0,steps)
omega=rep(0,steps)
num.pass=rep(0,steps)
alf=0.5 #alpha is fraction of establishment switching
prev.alpha=rep(0,steps) #new prevalence under "AC guidence" implementation
Se=rep(0,steps)
Sp=rep(0,steps)

for (i in 1:steps) {
  
  cut.off[i]=-0.05+0.05*i
  passing=subset(dat, ifelse(dat$frac.cen.pc>=cut.off[i],T,F))
  Se[i]=sum(ifelse(dat$frac.cen.pc>=cut.off[i],1,0)*dat$pass.ps)/sum(pass.prev)
  Sp[i]=sum(ifelse(dat$frac.cen.pc<cut.off[i],1,0)*(1-dat$pass.ps))/sum(1-pass.prev)
  num.pass[i]=length(passing$s.pc)
  failing=subset(dat, ifelse(dat$frac.cen.pc>cut.off[i],F,T))
  omega[i]=sum(passing$w)/sum(dat$w)
  n=ifelse(dat$n.m.s.pc+dat$s.pc==1,1.5,dat$n.m.s.pc+dat$s.pc) #add 1/2 for estab with 1 sample to do
  s=dat$s.pc
  w=dat$w
  param.pass[i,]=db_est(passing$s.pc,ifelse(passing$num.sam==1,1.5,passing$num.sam), passing$w/sum(passing$w))
  param.fail[i,]=db_est(failing$s.pc,ifelse(failing$num.sam==1,1.5,failing$num.sam), failing$w/sum(failing$w))
  prev.alpha[i]=(omega[i]+alf-alf*omega[i])*param.pass[i,1] + (1-omega[i])*(1-alf)*param.fail[i,1]
  #Se and Sp
  both.pass=sum(pass.apc*dat$pass.ps)
  print(cbind(cut.off[i],param.pass[i,1],param.fail[i,1], omega[i], num.pass[i]/num.estab,sum(passing$pass.ps)/length(passing$pass.ps)))
  
}   

plot(cut.off,param.pass[,1],ylab="prevalence in each stratum", xlab="Fraction of samples with no detected AC", ylim=c(0,0.1),col="darkgreen", 
main="Prevalence in passing and failing establishments
     that achieve an AC detecton standard" )
points(cut.off,param.fail[,1],pch=2,col="red" )
lines(cut.off,param.pass[,1],col="darkgreen")
lines(cut.off,param.fail[,1],col="red")
points(cut.off,prev.alpha,pch=3,col="purple" )
lines(cut.off,prev.alpha,col="purple")
abline(v=0.35, col="black")

abline(h=pop.params[1])
abline(h=0.75*pop.params[1], lty=2)
legend("topright",legend=c("passing prev.","failing prev.","alpha=0.5"),pch=c(1,2,3),col=c("darkgreen","red","purple"))

plot(cut.off,omega,ylab="fraction passing",xlab="fraction of samples with non detected AC",ylim=c(0,1),col="darkgreen", main="Fraction of production volume and establishments
     that are already meeting an AC detection goal" )
lines(cut.off,omega,col="green")
points(cut.off,num.pass/num.estab,pch=2,col="red" )
lines(cut.off,num.pass/num.estab,col="red" )
legend("bottomleft",legend=c("volume passing","prop. estabs passing"),pch=c(1,2),col=c("darkgreen","red"))

abline(v=.35, col="black")
cbind(cut.off,omega,num.pass/num.estab)

cut.off[order(Se+Sp)]
Se[order(Se+Sp)]+Sp[order(Se+Sp)]
cbind(cut.off,Se,Sp,Se+Sp)

plot(cut.off,Se, ylim=c(0,1), col="darkgreen",ylab="Se/Sp",
     xlab="fraction of samples with no detected AC", main="Performance charcteristics of performance standards 
     based on fractions of AC samples below the LOD")
lines(cut.off,Se, col="darkgreen")
points(cut.off,Sp, col="red")
lines(cut.off,Sp, col="red")
legend(0.025,0.25,legend=c("Se=passing both/passing ps","Sp=failing both/failing ps"),pch=c(1,2),col=c("darkgreen","red"), lty=c(1,1))
abline(v=.35, col="black")

